--[[ ArenaLive Spectator Functions: Spirit Healer Handler
Created by: Vadrak.
Creation Date: 20.07.2013
Last Update: "
This handler shows a spirit healer texture for dead players. It also tries to filter hunters with feign death.
The texture was created by my good friend Romina. Thanks to her for that!
]]--
local addonName, L = ...;

-- Create new Handler:
local SpiritHealer = ArenaLiveCore:AddHandler("SpiritHealer", "EventCore");
SpiritHealer:RegisterEvent("UNIT_AURA");

-- Create a table to store if a unit has recently used feign death:
local unitsFeigningDeath = {};

-- Max Buffs:
local MAX_BUFFS = 40;

local function OnUpdate(self, elapsed)

	local unit = self.unitFrame.unit;
	local guid = self.unitFrame.guid;
	
	if ( unit and guid ) then
		if ( UnitIsDeadOrGhost(unit) and not unitsFeigningDeath[guid] and not self.texture:IsShown() ) then
			self.texture:Show();
			self.flash:Show();
			self.flash:SetAlpha(1);
			self.alpha = 1;
			self.fadeIn = true;
		elseif ( not UnitIsDeadOrGhost(unit) and self.texture:IsShown() ) then
			self.flash:Hide();
			self.texture:Hide();
		end
	end

	if ( self.fadeIn ) then
		self.alpha = self.alpha - ( elapsed * 4);
		
		if ( self.alpha > 0 ) then
			if ( self.flash ) then
				self.flash:Show();
				self.flash:SetAlpha(self.alpha);
			end
		else
			self.flash:SetAlpha(0);
			self.fadeIn = nil;
		end
	end
end

local function Update(self)
	
	local unit = self.unitFrame.unit;
	local guid = self.unitFrame.guid;
	
	if ( unit and guid ) then
		if ( UnitIsDeadOrGhost(unit) and not unitsFeigningDeath[guid] and not self.texture:IsShown() ) then
			self.texture:Show();
			self.flash:Show();
			self.flash:SetAlpha(1);
			self.alpha = 1;
			self.fadeIn = true;
		elseif ( not UnitIsDeadOrGhost(unit) and self.texture:IsShown() ) then
			self.flash:Hide();
			self.texture:Hide();
		end
	else
		self:Reset();
	end

end

local function Reset(self)
	
	self.flash:Hide();
	self.texture:Hide();

end



function SpiritHealer:AddFrame(spiritHealer, texture, flash, frameLevelRaise, unitFrame)

	-- Create a reference for the spirit healher frame inside the unit frame and vice versa.
	unitFrame.spiritHealer = spiritHealer;
	spiritHealer.unitFrame = unitFrame;	

	-- Register the ccIndcator in the unit frame's handler list:
	unitFrame.handlerList.spiritHealer = true;	

	-- Set texture reference:
	spiritHealer.texture = texture;	
	spiritHealer.flash = flash;
	
	-- Basic values:
	spiritHealer.alpha = 0;
	
	-- Set the basic functions for the frame:
	spiritHealer.OnUpdate = OnUpdate;
	spiritHealer.Update = Update;
	spiritHealer.Reset = Reset;	
	
	if ( frameLevelRaise > 0 ) then
		spiritHealer:SetFrameLevel(spiritHealer:GetFrameLevel() + frameLevelRaise);
	end
	
	-- Set Script:
	spiritHealer:SetScript("OnUpdate", spiritHealer.OnUpdate);
end

function SpiritHealer:OnEvent(event, ...)

	if ( event == "UNIT_AURA" ) then
	
		local unit = ...;
		local guid = UnitGUID(unit);
		local isFeigningDeath = false;
		local ranOnce = false;
		for i = 1, MAX_BUFFS do
			local _, _, _, _, _, _, _, _, _, _, spellID = UnitBuff(unit, i);
			
			-- SpellID for Feign Death:
			if ( spellID ) then
				if ( i == 1 ) then
					ranOnce = true;
				end
				
				if ( spellID == 5384 ) then
					isFeigningDeath = true;
				end
			
			else
				break;
			end
		end
		
		if ( ranOnce ) then
			unitsFeigningDeath[guid] = isFeigningDeath;
		end
	end

end